import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.String.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileNameExtensionFilter;
import java.text.*;
import java.util.Arrays;
import java.util.ArrayList;
/**
 * Program to break up SaGa Frontier battle scripts by their pointers.
 * 
 * ScriptHelper (c) 2016 - 2017
 * https://bitbucket.org/mithridates
 */
public class ScriptHelper extends JFrame
{
    private Container contentPane;

    private JMenuBar menuBar;
    private JMenu scriptMenu;
    private JMenuItem openFile;
    private JMenuItem parsePointers;

    private SFScript[] scripts;
    /**
     * Constructor for objects of class ScriptHelper
     */
    public ScriptHelper()
    {
        super("ScriptHelper");
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setPreferredSize(new Dimension(250, 120));//Works on Linux

        contentPane = getContentPane();

        makeMenu();

        pack();
        setVisible(true);

        //setResizable(false);//Doesn't matter right now

        openFile.setEnabled(true);
        parsePointers.setEnabled(false);
    }

    /**
     * Method to set up menu.
     */
    public void makeMenu()
    {
        menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        scriptMenu = new JMenu("Scripts");
        menuBar.add(scriptMenu);

        openFile = new JMenuItem("Open File...");
        openFile.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {//anonymous inner class, saves space
                    JFileChooser openFileChooser = new JFileChooser();

                    FileNameExtensionFilter eventFilter = new 
                        FileNameExtensionFilter("EVENT.BIN or EVENT2.BIN only",
                            "bin");

                    openFileChooser.setFileFilter(eventFilter);

                    openFileChooser.setCurrentDirectory(new File(
                            System.getProperty("user.dir")));

                    int validFile = openFileChooser.showOpenDialog(ScriptHelper.this);

                    if (validFile == JFileChooser.APPROVE_OPTION)
                    {
                        packScripts(openFileChooser.getSelectedFile());
                        openFile.setEnabled(true);
                        parsePointers.setEnabled(true);
                    }
                }
            });
        scriptMenu.add(openFile);

        parsePointers = new JMenuItem("Parse scripts by pointers");
        parsePointers.addActionListener(new ActionListener()
            {//anonymous inner class, saves space
                public void actionPerformed(ActionEvent evt)
                {
                    JFileChooser parseChooser = new JFileChooser();

                    FileNameExtensionFilter parseFilter = new
                        FileNameExtensionFilter("Plain text file", "txt");

                    parseChooser.setFileFilter(parseFilter);

                    parseChooser.setCurrentDirectory(new File(
                            System.getProperty("user.dir")));

                    int validFile = parseChooser.showSaveDialog(ScriptHelper.this);

                    if (validFile == JFileChooser.APPROVE_OPTION)
                    {
                        parseScripts(parseChooser.getSelectedFile());
                    }
                }
            });
        scriptMenu.add(parsePointers);
    }

    /**
     * Method to load pointers and scripts into a data structure.
     * 
     * @param File battleScriptFile
     */
    private void packScripts(File battleScriptFile)
    {
        RandomAccessFile scriptFile = null;
        try
        {
            scriptFile = new RandomAccessFile(
                battleScriptFile, "r");

            scripts = new SFScript[128];

            //first, get scriptPointers
            for (int i = 0; i < scripts.length; i++)
            {
                int scriptPointer = scriptFile.readUnsignedByte();
                scriptPointer = (scriptFile.readUnsignedByte() << 8) 
                | scriptPointer;
                //^must account for endianness: shift bits and logical OR^

                scripts[i] = new SFScript(scriptPointer);
            }

            //next, get scripts
            for (int i = 0; i < scripts.length; i++)
            {
                int length = 0;
                int[] script;
                if (i < (scripts.length - 1))
                {//for all but last SFScript, get length in this way
                    length = scripts[i + 1].getScriptPointer() - 
                    scripts[i].getScriptPointer();
                    script = new int[length];
                }
                else
                {//last SFScript must use lengh of scriptFile instead 
                    length = (int)scriptFile.length() - 
                    scripts[i].getScriptPointer();
                    script = new int[length];
                }
                scriptFile.seek((long)scripts[i].getScriptPointer());

                for (int j = 0; j < length; j++)
                {
                    script[j] = scriptFile.readUnsignedByte();
                }
                scripts[i].setScript(script);
            }
        }
        catch (Exception e)
        {
            System.out.println(e);
            e.printStackTrace();
        }
        finally
        {
            try
            {
                scriptFile.close();
            }
            catch (Exception e)
            {
                ;//do nothing
            }
        }
    }

    /**
     * Method to write pointers and scripts to a given text file.
     * 
     * @param File textFile
     */
    private void parseScripts(File textFile)
    {
        BufferedWriter bWriter = null;
        try
        {
            bWriter = new BufferedWriter(
                new FileWriter(textFile));
            int counter = 1;
            for (SFScript scr : scripts)
            {
                if (scr.getScriptPointer() < 4096)
                {//if only three hex digits, add leading zero
                    bWriter.write("Pointer " + String.valueOf(counter) + ": 0" +
                        String.valueOf(Integer.toHexString(
                                scr.getScriptPointer()).toUpperCase()));
                }
                else
                {//else no need for leading digit, zero or otherwise
                    bWriter.write("Pointer " + String.valueOf(counter) + ": " +
                        String.valueOf(Integer.toHexString(
                                scr.getScriptPointer()).toUpperCase()));
                }
                bWriter.newLine();
                bWriter.newLine();
                //^write the pointer and two newlines^

                counter ++;

                int[] tempScript = scr.getScript();
                for (int i = 0; i < tempScript.length; i++)
                {
                    if (tempScript[i] < 16)
                    {//if only one hex digit, add leading zero
                        bWriter.write("0" + String.valueOf(
                                Integer.toHexString(
                                    tempScript[i]).toUpperCase()) + " ");
                    }
                    else
                    {//else no need for leading digit, zero or otherwise
                        bWriter.write(String.valueOf(
                                Integer.toHexString(
                                    tempScript[i]).toUpperCase()) + " ");
                    }
                }
                //^write the script itself^

                bWriter.newLine();
                bWriter.newLine();
            }

        }
        catch (Exception e)
        {
            System.out.println(e);
            e.printStackTrace();
        }
        finally
        {
            if (bWriter != null)
            {
                try
                {
                    bWriter.close();
                }
                catch (Exception e)
                {
                    ;//do nothing
                }
            }
        }
    }
}
